@@ -51,6 +51,7 @@
 #include <X11/Xatom.h>
 #include <X11/Xos.h>
 
+#include "zvt-marshal.h"
 
 /* define to 'x' to enable copious debug output */
 #define d(x)
@@ -78,13 +79,14 @@ static gint zvt_term_motion_notify (GtkWidget *widget, GdkEventMotion *event);
 static gint zvt_term_key_press (GtkWidget *widget, GdkEventKey *event);
 static gint zvt_term_focus_in (GtkWidget *widget, GdkEventFocus *event);
 static gint zvt_term_focus_out (GtkWidget *widget, GdkEventFocus *event);
+static gint zvt_term_scroll_event (GtkWidget *widget, GdkEventScroll *event);
 static gint zvt_term_selection_clear (GtkWidget *widget, GdkEventSelection *event);
 static void zvt_term_selection_get (GtkWidget *widget,
 				    GtkSelectionData *selection_data_ptr, 
 				    guint info, 
 				    guint time);
 static void zvt_term_child_died (ZvtTerm *term);
-static void zvt_term_title_changed (ZvtTerm *term, char *str, VTTITLE_TYPE type);
+static void zvt_term_title_changed (ZvtTerm *term, VTTITLE_TYPE type, const char *str);
 static void zvt_term_title_changed_raise (void *user_data, char *str, VTTITLE_TYPE type);
 static void zvt_term_got_output (ZvtTerm *term, const gchar *buffer, gint count);
 static gint zvt_term_cursor_blink (gpointer data);
@@ -202,10 +204,10 @@ zvt_term_class_init (ZvtTermClass *class)
 		    /*
                     gnome_marshal_VOID__INT_POINTER,
 		    */
-                    gtk_marshal_VOID__POINTER_INT,
+                    zvt_marshal_VOID__INT_POINTER,
                     GTK_TYPE_NONE, 2,
-		    GTK_TYPE_POINTER,
-		    GTK_TYPE_INT);
+                    GTK_TYPE_INT,
+		    GTK_TYPE_POINTER);
   
   term_signals[GOT_OUTPUT] =
       gtk_signal_new ("got_output",
@@ -268,6 +270,7 @@ zvt_term_class_init (ZvtTermClass *class)
   widget_class->button_press_event = zvt_term_button_press;
   widget_class->button_release_event = zvt_term_button_release;
   widget_class->motion_notify_event = zvt_term_motion_notify;
+  widget_class->scroll_event = zvt_term_scroll_event;
 
   widget_class->selection_clear_event = zvt_term_selection_clear;
   widget_class->selection_get = zvt_term_selection_get;
@@ -549,8 +552,6 @@ term_force_size(ZvtTerm *term)
 				    GTK_WIDGET(term),
 				    &hints,
 				    GDK_HINT_RESIZE_INC|GDK_HINT_MIN_SIZE|GDK_HINT_BASE_SIZE);
-    } else {
-      gtk_widget_queue_resize (GTK_WIDGET (term));
     }
 
     d(printf("forcing term size to %d, %d\n", term->grid_width, term->grid_height));
@@ -777,7 +778,24 @@ zvt_term_set_color_scheme (ZvtTerm *term, gushort *red, gushort *grn, gushort *b
   gdk_window_set_background (GTK_WIDGET (term)->window, &c);
   gdk_window_clear (GTK_WIDGET (term)->window);
 #endif
+  gtk_widget_queue_draw (GTK_WIDGET (term));
 
+  /* Need to unset term->back_last so the GC's get updated properly
+   */
+  term->back_last = -1;
+
+  /* If we don't have a background pixmap, we need to update the
+   * background gc.
+   */
+  if (!zp->background || zp->background->type == ZVT_BGTYPE_NONE) {
+         if (term->back_gc) {
+	        GdkColor pen;
+
+		pen.pixel = term->colors[17].pixel;
+		gdk_gc_set_foreground (term->back_gc, &pen);
+	  }
+  }
+  
   /* always clear up any old queued values */
   clone_col(&zp->queue_red, 0);
   clone_col(&zp->queue_green, 0);
@@ -827,7 +845,8 @@ zvt_term_realize (GtkWidget *widget)
   GdkPixmap *cursor_dot_pm;
   gint attributes_mask;
   struct _zvtprivate *zp;
-
+  GdkColor c;
+  
   g_return_if_fail (widget != NULL);
   g_return_if_fail (ZVT_IS_TERM (widget));
 
@@ -859,7 +878,8 @@ zvt_term_realize (GtkWidget *widget)
   widget->style = gtk_style_attach (widget->style, widget->window);
   gdk_window_set_user_data (widget->window, widget);
 #if 1
-  gtk_style_set_background (widget->style, widget->window, GTK_STATE_ACTIVE);
+  c.pixel = term->colors [17].pixel;  
+  gdk_window_set_background (widget->window, &c);
 #else
   gdk_window_set_back_pixmap(widget->window, 0, 1);
 #endif
@@ -1089,19 +1109,11 @@ zvt_term_focus_out(GtkWidget *widget, GdkEventFocus *event)
   return FALSE;
 }
 
-/*
- Ok, this is a bit weird, we basically keep returning
- the initial size, until we no longer need it (this is
- during realization), then we just return 1x1 always,
- so that the window can grow or shrink so that set_usize
- always works.
-*/
 static void 
 zvt_term_size_request (GtkWidget      *widget,
 		       GtkRequisition *requisition)
 {
   ZvtTerm *term;
-  guint grid_width, grid_height;
 
   g_return_if_fail (widget != NULL);
   g_return_if_fail (ZVT_IS_TERM (widget));
@@ -1109,19 +1121,8 @@ zvt_term_size_request (GtkWidget      *widget,
 
   term = ZVT_TERM (widget);
 
-  grid_width = term->grid_width;
-  grid_height = term->grid_height;
-  
-  if (grid_width <= 0)
-    grid_width = 8;
-  
-  if (grid_height <= 0)
-    grid_height = 2;
-
-  requisition->width = (grid_width * term->charwidth) + 
-    (widget->style->xthickness * 2) + PADDING;
-  requisition->height = (grid_height * term->charheight) + 
-    (widget->style->ythickness * 2);
+  requisition->width = term->charwidth + widget->style->xthickness * 2 + PADDING;
+  requisition->height = term->charheight + widget->style->ythickness * 2;
 
   /* debug ouput */
   d( printf("zvt_term_size_request x=%d y=%d\n", grid_width, grid_height) );
@@ -1223,9 +1224,6 @@ zvt_term_expose (GtkWidget      *widget,
     /* FIXME: may update 1 more line/char than needed */
     term->in_expose = 1;
 
-    /*
-      this could probably do what draw does, but it looks a bit slower if you do.
-     */
     if(zp->background) {
       gdk_draw_rectangle (widget->window,
 			  term->back_gc, 1,
@@ -1381,7 +1379,7 @@ zvt_term_set_fonts_internal(ZvtTerm *term, GdkFont *font, GdkFont *font_bold)
 				   1, term->charheight, depth);
   }
 
-  /*gtk_widget_queue_resize (GTK_WIDGET (term));*/
+  gtk_widget_queue_resize (GTK_WIDGET (term));
 }
 
 /**
@@ -1402,11 +1400,11 @@ zvt_term_set_fonts (ZvtTerm *term, GdkFont *font, GdkFont *font_bold)
   g_return_if_fail (ZVT_IS_TERM (term));
   g_return_if_fail (font != NULL);
 
-  zvt_term_set_fonts_internal (term, font, font_bold);
-
   gdk_font_ref (font);
   if (font_bold)
       gdk_font_ref (font_bold);
+  
+  zvt_term_set_fonts_internal (term, font, font_bold);
 }
 
 /**
@@ -1697,15 +1695,8 @@ zvt_term_button_press (GtkWidget      *widget,
       }
     }
     break;
-
-  case 4:
-    zvt_term_scroll_by_lines (term, -12);
-    break;
-
-  case 5:
-    zvt_term_scroll_by_lines (term, 12);
-    break;
   }
+
   return FALSE;
 }
 
@@ -1802,15 +1793,14 @@ zvt_term_button_release (GtkWidget      *widget,
 static void
 zvt_term_scroll_by_lines (ZvtTerm *term, int n)
 {
+  GtkAdjustment *adj = term->adjustment;
   gfloat new_value = 0;
 
-  if (n > 0)
-    new_value = MIN (term->adjustment->value + n, term->adjustment->upper - n);
-  else if (n < 0)
-    new_value = MAX (term->adjustment->value + n, term->adjustment->lower);
-  else
+  if (n == 0)
     return;
 
+  new_value = CLAMP (adj->value + n, adj->lower, adj->upper - adj->page_size);
+  
   gtk_adjustment_set_value (term->adjustment, new_value);
 }
 
@@ -1830,6 +1820,28 @@ static gint zvt_selectscroll(gpointer data)
   return TRUE;
 }
 
+static gint zvt_term_scroll_event (GtkWidget *widget, GdkEventScroll *event)
+{
+  ZvtTerm *term;
+  
+  g_return_val_if_fail (ZVT_IS_TERM (widget), FALSE);
+  g_return_val_if_fail (event != NULL, FALSE);
+  
+  term = ZVT_TERM (widget);
+  
+  if (event->direction == GDK_SCROLL_UP) {
+    zvt_term_scroll_by_lines (term, -12);
+    return TRUE;
+  }
+  
+  if (event->direction == GDK_SCROLL_DOWN) {
+    zvt_term_scroll_by_lines (term, 12);
+    return TRUE;
+  }
+  
+  return FALSE;
+}
+
 /*
   mouse motion notify.
   only gets called for the first motion?  why?
@@ -2302,7 +2314,7 @@ static void
 zvt_term_scroll (ZvtTerm *term, int n)
 {
   gfloat new_value = 0;
-  
+
   if (n) {
     new_value = term->adjustment->value + (n * term->adjustment->page_size);
   } else if (new_value == (term->adjustment->upper - term->adjustment->page_size)) {
@@ -2575,7 +2587,7 @@ zvt_term_child_died (ZvtTerm *term)
 
 /* dummy default signal handler for title_changed */
 static void
-zvt_term_title_changed (ZvtTerm *term, char *str, VTTITLE_TYPE type)
+zvt_term_title_changed (ZvtTerm *term, VTTITLE_TYPE type, const char *str)
 {
   g_return_if_fail (term != NULL);
   g_return_if_fail (ZVT_IS_TERM (term));
@@ -3338,7 +3350,7 @@ vt_scroll_area(void *user_data, int firstrow, int count, int offset, int fill)
     gdk_draw_rectangle(widget->window,
 		       gc,
 		       1,
-		       offx, offy+(firstrow+offset)*term->charheight,
+ 		       offx, offy+(firstrow+offset)*term->charheight,
 		       term->vx->vt.width*term->charwidth, (-offset)*term->charheight);
   }
 
